# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["Accounting"]


class Accounting(BaseModel):
    account_id: Optional[str] = None
    """The ID of one of your accounting categories.

    Note that these will only be accessible if your accounting system has been
    connected.
    """

    class_id: Optional[str] = None
    """The ID of one of the class objects in your accounting system.

    Class objects track segments of your business independent of client or project.
    Note that these will only be accessible if your accounting system has been
    connected.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import (
    PaymentActionListResponse,
    PaymentActionCreateResponse,
    PaymentActionUpdateResponse,
    PaymentActionRetrieveResponse,
)
from modern_treasury._utils import parse_datetime
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPaymentActions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        payment_action = client.payment_actions.create(
            type="type",
        )
        assert_matches_type(PaymentActionCreateResponse, payment_action, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        payment_action = client.payment_actions.create(
            type="type",
            actionable_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            actionable_type="actionable_type",
            details={},
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PaymentActionCreateResponse, payment_action, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.payment_actions.with_raw_response.create(
            type="type",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_action = response.parse()
        assert_matches_type(PaymentActionCreateResponse, payment_action, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.payment_actions.with_streaming_response.create(
            type="type",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_action = response.parse()
            assert_matches_type(PaymentActionCreateResponse, payment_action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        payment_action = client.payment_actions.retrieve(
            "id",
        )
        assert_matches_type(PaymentActionRetrieveResponse, payment_action, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.payment_actions.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_action = response.parse()
        assert_matches_type(PaymentActionRetrieveResponse, payment_action, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.payment_actions.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_action = response.parse()
            assert_matches_type(PaymentActionRetrieveResponse, payment_action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.payment_actions.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        payment_action = client.payment_actions.update(
            id="id",
            status="pending",
        )
        assert_matches_type(PaymentActionUpdateResponse, payment_action, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.payment_actions.with_raw_response.update(
            id="id",
            status="pending",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_action = response.parse()
        assert_matches_type(PaymentActionUpdateResponse, payment_action, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.payment_actions.with_streaming_response.update(
            id="id",
            status="pending",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_action = response.parse()
            assert_matches_type(PaymentActionUpdateResponse, payment_action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.payment_actions.with_raw_response.update(
                id="",
                status="pending",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        payment_action = client.payment_actions.list()
        assert_matches_type(SyncPage[PaymentActionListResponse], payment_action, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        payment_action = client.payment_actions.list(
            actionable_id="actionable_id",
            actionable_type="actionable_type",
            after_cursor="after_cursor",
            created_at={
                "eq": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            internal_account_id="internal_account_id",
            metadata={"foo": "string"},
            per_page=0,
            status="pending",
            type="evolve_non_processing_transaction",
        )
        assert_matches_type(SyncPage[PaymentActionListResponse], payment_action, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.payment_actions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_action = response.parse()
        assert_matches_type(SyncPage[PaymentActionListResponse], payment_action, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.payment_actions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_action = response.parse()
            assert_matches_type(SyncPage[PaymentActionListResponse], payment_action, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncPaymentActions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        payment_action = await async_client.payment_actions.create(
            type="type",
        )
        assert_matches_type(PaymentActionCreateResponse, payment_action, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        payment_action = await async_client.payment_actions.create(
            type="type",
            actionable_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            actionable_type="actionable_type",
            details={},
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PaymentActionCreateResponse, payment_action, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.payment_actions.with_raw_response.create(
            type="type",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_action = response.parse()
        assert_matches_type(PaymentActionCreateResponse, payment_action, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.payment_actions.with_streaming_response.create(
            type="type",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_action = await response.parse()
            assert_matches_type(PaymentActionCreateResponse, payment_action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        payment_action = await async_client.payment_actions.retrieve(
            "id",
        )
        assert_matches_type(PaymentActionRetrieveResponse, payment_action, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.payment_actions.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_action = response.parse()
        assert_matches_type(PaymentActionRetrieveResponse, payment_action, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.payment_actions.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_action = await response.parse()
            assert_matches_type(PaymentActionRetrieveResponse, payment_action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.payment_actions.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        payment_action = await async_client.payment_actions.update(
            id="id",
            status="pending",
        )
        assert_matches_type(PaymentActionUpdateResponse, payment_action, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.payment_actions.with_raw_response.update(
            id="id",
            status="pending",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_action = response.parse()
        assert_matches_type(PaymentActionUpdateResponse, payment_action, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.payment_actions.with_streaming_response.update(
            id="id",
            status="pending",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_action = await response.parse()
            assert_matches_type(PaymentActionUpdateResponse, payment_action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.payment_actions.with_raw_response.update(
                id="",
                status="pending",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        payment_action = await async_client.payment_actions.list()
        assert_matches_type(AsyncPage[PaymentActionListResponse], payment_action, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        payment_action = await async_client.payment_actions.list(
            actionable_id="actionable_id",
            actionable_type="actionable_type",
            after_cursor="after_cursor",
            created_at={
                "eq": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            internal_account_id="internal_account_id",
            metadata={"foo": "string"},
            per_page=0,
            status="pending",
            type="evolve_non_processing_transaction",
        )
        assert_matches_type(AsyncPage[PaymentActionListResponse], payment_action, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.payment_actions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment_action = response.parse()
        assert_matches_type(AsyncPage[PaymentActionListResponse], payment_action, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.payment_actions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment_action = await response.parse()
            assert_matches_type(AsyncPage[PaymentActionListResponse], payment_action, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import date
from typing_extensions import Literal

from ..._models import BaseModel
from ..bank_settings import BankSettings
from .identification_create_request import IdentificationCreateRequest
from ..wealth_and_employment_details import WealthAndEmploymentDetails
from .legal_entity_compliance_detail import LegalEntityComplianceDetail
from .legal_entity_address_create_request import LegalEntityAddressCreateRequest
from .legal_entity_industry_classification import LegalEntityIndustryClassification

__all__ = ["ChildLegalEntityCreate", "PhoneNumbers", "PhoneNumber"]


class PhoneNumber(BaseModel):
    phone_number: Optional[str] = None


PhoneNumbers = PhoneNumber
"""This type is deprecated and will be removed in a future release.

Please use PhoneNumber instead.
"""


class ChildLegalEntityCreate(BaseModel):
    addresses: Optional[List[LegalEntityAddressCreateRequest]] = None
    """A list of addresses for the entity."""

    bank_settings: Optional[BankSettings] = None

    business_name: Optional[str] = None
    """The business's legal business name."""

    citizenship_country: Optional[str] = None
    """The country of citizenship for an individual."""

    compliance_details: Optional[LegalEntityComplianceDetail] = None

    date_formed: Optional[date] = None
    """A business's formation date (YYYY-MM-DD)."""

    date_of_birth: Optional[date] = None
    """An individual's date of birth (YYYY-MM-DD)."""

    doing_business_as_names: Optional[List[str]] = None

    email: Optional[str] = None
    """The entity's primary email."""

    first_name: Optional[str] = None
    """An individual's first name."""

    identifications: Optional[List[IdentificationCreateRequest]] = None
    """A list of identifications for the legal entity."""

    industry_classifications: Optional[List[LegalEntityIndustryClassification]] = None
    """A list of industry classifications for the legal entity."""

    last_name: Optional[str] = None
    """An individual's last name."""

    legal_entity_type: Optional[Literal["business", "individual"]] = None
    """The type of legal entity."""

    legal_structure: Optional[
        Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
    ] = None
    """The business's legal structure."""

    metadata: Optional[Dict[str, str]] = None
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    middle_name: Optional[str] = None
    """An individual's middle name."""

    phone_numbers: Optional[List[PhoneNumber]] = None

    politically_exposed_person: Optional[bool] = None
    """Whether the individual is a politically exposed person."""

    preferred_name: Optional[str] = None
    """An individual's preferred name."""

    prefix: Optional[str] = None
    """An individual's prefix."""

    risk_rating: Optional[Literal["low", "medium", "high"]] = None
    """The risk rating of the legal entity. One of low, medium, high."""

    suffix: Optional[str] = None
    """An individual's suffix."""

    wealth_and_employment_details: Optional[WealthAndEmploymentDetails] = None

    website: Optional[str] = None
    """The entity's primary website URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .bank_settings_param import BankSettingsParam
from .wealth_and_employment_details_param import WealthAndEmploymentDetailsParam
from .shared_params.identification_create_request import IdentificationCreateRequest
from .shared_params.legal_entity_compliance_detail import LegalEntityComplianceDetail
from .shared_params.legal_entity_address_create_request import LegalEntityAddressCreateRequest
from .shared_params.legal_entity_industry_classification import LegalEntityIndustryClassification

__all__ = ["LegalEntityUpdateParams", "PhoneNumbers", "PhoneNumber"]


class LegalEntityUpdateParams(TypedDict, total=False):
    addresses: Iterable[LegalEntityAddressCreateRequest]
    """A list of addresses for the entity."""

    bank_settings: Optional[BankSettingsParam]

    business_name: Optional[str]
    """The business's legal business name."""

    citizenship_country: Optional[str]
    """The country of citizenship for an individual."""

    compliance_details: Optional[LegalEntityComplianceDetail]

    date_formed: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """A business's formation date (YYYY-MM-DD)."""

    date_of_birth: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """An individual's date of birth (YYYY-MM-DD)."""

    doing_business_as_names: List[str]

    email: Optional[str]
    """The entity's primary email."""

    first_name: Optional[str]
    """An individual's first name."""

    identifications: Iterable[IdentificationCreateRequest]
    """A list of identifications for the legal entity."""

    industry_classifications: Iterable[LegalEntityIndustryClassification]
    """A list of industry classifications for the legal entity."""

    last_name: Optional[str]
    """An individual's last name."""

    legal_structure: Optional[
        Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
    ]
    """The business's legal structure."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    middle_name: Optional[str]
    """An individual's middle name."""

    phone_numbers: Iterable[PhoneNumber]

    politically_exposed_person: Optional[bool]
    """Whether the individual is a politically exposed person."""

    preferred_name: Optional[str]
    """An individual's preferred name."""

    prefix: Optional[str]
    """An individual's prefix."""

    risk_rating: Optional[Literal["low", "medium", "high"]]
    """The risk rating of the legal entity. One of low, medium, high."""

    suffix: Optional[str]
    """An individual's suffix."""

    wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam]

    website: Optional[str]
    """The entity's primary website URL."""


class PhoneNumber(TypedDict, total=False):
    phone_number: str


PhoneNumbers = PhoneNumber
"""This type is deprecated and will be removed in a future release.

Please use PhoneNumber instead.
"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .address import Address as Address
from .currency import Currency as Currency
from .accounting import Accounting as Accounting
from .accounts_type import AccountsType as AccountsType
from .async_response import AsyncResponse as AsyncResponse
from .contact_detail import ContactDetail as ContactDetail
from .ledger_balance import LedgerBalance as LedgerBalance
from .address_request import AddressRequest as AddressRequest
from .ledger_balances import LedgerBalances as LedgerBalances
from .foreign_exchange_rate import ForeignExchangeRate as ForeignExchangeRate
from .transaction_direction import TransactionDirection as TransactionDirection
from .child_legal_entity_create import ChildLegalEntityCreate as ChildLegalEntityCreate
from .ledger_entry_create_request import LedgerEntryCreateRequest as LedgerEntryCreateRequest
from .identification_create_request import IdentificationCreateRequest as IdentificationCreateRequest
from .ledger_account_create_request import LedgerAccountCreateRequest as LedgerAccountCreateRequest
from .legal_entity_compliance_detail import LegalEntityComplianceDetail as LegalEntityComplianceDetail
from .ledger_transaction_create_request import LedgerTransactionCreateRequest as LedgerTransactionCreateRequest
from .legal_entity_address_create_request import LegalEntityAddressCreateRequest as LegalEntityAddressCreateRequest
from .legal_entity_industry_classification import LegalEntityIndustryClassification as LegalEntityIndustryClassification
